@@ -22,22 +22,46 @@ import inspect
 import json
 import logging
 import re
-import typing  # type: ignore
+import typing  # noqa: F401
 import warnings
 from copy import copy
 from io import StringIO, TextIOWrapper
 from json import JSONEncoder
-from typing import Any, Callable, Dict, Iterable, List, Optional, Set, Tuple, Type, TypeVar, Union, cast
+from sys import version_info
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 from xml.etree import ElementTree
 
-from .formatters import CurrentFormatter
+if version_info >= (3, 8):
+    from typing import Protocol
+else:
+    from typing_extensions import Protocol  # type: ignore[misc]
+
+from .formatters import BaseNameFormatter, CurrentFormatter
 from .helpers import BaseHelper
 
 logger = logging.getLogger('serializable')
 logger.setLevel(logging.INFO)
 
+
+class _Klass(Protocol):
+    __qualname__: str
+
+
 _F = TypeVar("_F", bound=Callable[..., Any])
-_T = TypeVar('_T')
+_T = TypeVar('_T', bound=_Klass)
 
 
 @enum.unique
@@ -225,19 +249,26 @@ def _from_json(cls: Type[_T], data: Dict[str, Any]) -> object:
     return cls(**_data)
 
 
-def _as_xml(self: _T, as_string: bool = True, element_name: Optional[str] = None) -> Union[ElementTree.Element, str]:
+def _as_xml(self: _T, as_string: bool = True, element_name: Optional[str] = None,
+            xmlns: Optional[str] = None) -> Union[ElementTree.Element, str]:
     logging.debug(f'Dumping {self} to XML...')
 
     this_e_attributes = {}
     klass_qualified_name = f'{self.__module__}.{self.__class__.__qualname__}'
-    serializable_property_info = ObjectMetadataLibrary.klass_property_mappings.get(klass_qualified_name, {})
+    serializable_property_info = {k: v for k, v in sorted(
+        ObjectMetadataLibrary.klass_property_mappings.get(klass_qualified_name, {}).items(),
+        key=lambda i: i[1].xml_sequence)}
 
-    # Handle any Properties that should be attributes
     for k, v in self.__dict__.items():
         # Remove leading _ in key names
         new_key = k[1:]
         if new_key.startswith('_') or '__' in new_key:
             continue
+        new_key = BaseNameFormatter.decode_handle_python_builtins_and_keywords(name=new_key)
+
+        # Ignore Nones
+        if not v:
+            continue
 
         if new_key in serializable_property_info:
             prop_info = serializable_property_info.get(new_key)
@@ -248,78 +279,80 @@ def _as_xml(self: _T, as_string: bool = True, element_name: Optional[str] = None
 
                 if prop_info.custom_type and prop_info.is_helper_type():
                     v = prop_info.custom_type.serialize(v)
+                elif prop_info.is_enum:
+                    v = v.value
 
                 this_e_attributes.update({new_key: str(v)})
 
-    this_e = ElementTree.Element(
-        element_name or CurrentFormatter.formatter.encode(self.__class__.__name__), this_e_attributes
-    )
+    element_name = _namespace_element_name(tag_name=element_name,
+                                           xmlns=xmlns) if element_name else _namespace_element_name(
+        tag_name=CurrentFormatter.formatter.encode(self.__class__.__name__), xmlns=xmlns)
+    this_e = ElementTree.Element(element_name, this_e_attributes)
 
     # Handle remaining Properties that will be sub elements
-    for k, v in self.__dict__.items():
+    for k, prop_info in serializable_property_info.items():
+        v = getattr(self, k)
+
         # Ignore None values by default
         if v is None:
             continue
 
-        # Remove leading _ in key names
-        new_key = k[1:]
-        if new_key.startswith('_') or '__' in new_key:
-            continue
+        new_key = BaseNameFormatter.decode_handle_python_builtins_and_keywords(name=k)
 
-        if new_key in serializable_property_info:
-            prop_info = serializable_property_info.get(new_key)
-            if not prop_info:
-                raise ValueError(f'{new_key} is not a known Property for {klass_qualified_name}')
+        if not prop_info:
+            raise ValueError(f'{new_key} is not a known Property for {klass_qualified_name}')
 
-            if not prop_info.is_xml_attribute:
-                new_key = prop_info.custom_names.get(SerializationType.XML, new_key)
+        if not prop_info.is_xml_attribute:
+            new_key = prop_info.custom_names.get(SerializationType.XML, new_key)
 
-                if new_key == '.':
-                    this_e.text = str(v)
-                    continue
+            if new_key == '.':
+                this_e.text = str(v)
+                continue
 
-                if CurrentFormatter.formatter:
-                    new_key = CurrentFormatter.formatter.encode(property_name=new_key)
+            if CurrentFormatter.formatter:
+                new_key = CurrentFormatter.formatter.encode(property_name=new_key)
+            new_key = _namespace_element_name(tag_name=new_key, xmlns=xmlns)
 
-                if prop_info.custom_type:
-                    if prop_info.is_helper_type():
-                        ElementTree.SubElement(this_e, new_key).text = str(prop_info.custom_type.serialize(v))
-                    else:
-                        ElementTree.SubElement(this_e, new_key).text = str(prop_info.custom_type(v))
-                elif prop_info.is_array and prop_info.xml_array_config:
-                    _array_type, nested_key = prop_info.xml_array_config
-                    if _array_type and _array_type == XmlArraySerializationType.NESTED:
-                        nested_e = ElementTree.SubElement(this_e, new_key)
-                    else:
-                        nested_e = this_e
-
-                    for j in v:
-                        if not prop_info.is_primitive_type():
-                            nested_e.append(j.as_xml(as_string=False, element_name=nested_key))
-                        elif prop_info.concrete_type() in (float, int):
-                            ElementTree.SubElement(nested_e, nested_key).text = str(j)
-                        elif prop_info.concrete_type() is bool:
-                            ElementTree.SubElement(nested_e, nested_key).text = str(j).lower()
-                        else:
-                            # Assume type is str
-                            ElementTree.SubElement(nested_e, nested_key).text = str(j)
-                elif prop_info.is_enum:
-                    ElementTree.SubElement(this_e, new_key).text = str(v.value)
-                elif not prop_info.is_primitive_type():
-                    global_klass_name = f'{prop_info.concrete_type.__module__}.{prop_info.concrete_type.__name__}'
-                    if global_klass_name in ObjectMetadataLibrary.klass_mappings:
-                        # Handle other Serializable Classes
-                        this_e.append(v.as_xml(as_string=False, element_name=new_key))
-                    else:
-                        # Handle properties that have a type that is not a Python Primitive (e.g. int, float, str)
-                        ElementTree.SubElement(this_e, new_key).text = str(v)
-                elif prop_info.type_ in (float, int):
-                    ElementTree.SubElement(this_e, new_key).text = str(v)
-                elif prop_info.type_ is bool:
-                    ElementTree.SubElement(this_e, new_key).text = str(v).lower()
+            if prop_info.custom_type:
+                if prop_info.is_helper_type():
+                    ElementTree.SubElement(this_e, new_key).text = str(prop_info.custom_type.serialize(v))
                 else:
-                    # Assume type is str
+                    ElementTree.SubElement(this_e, new_key).text = str(prop_info.custom_type(v))
+            elif prop_info.is_array and prop_info.xml_array_config:
+                _array_type, nested_key = prop_info.xml_array_config
+                nested_key = _namespace_element_name(tag_name=nested_key, xmlns=xmlns)
+                if _array_type and _array_type == XmlArraySerializationType.NESTED and len(v) > 0:
+                    nested_e = ElementTree.SubElement(this_e, new_key)
+                else:
+                    nested_e = this_e
+
+                for j in v:
+                    if not prop_info.is_primitive_type():
+                        nested_e.append(j.as_xml(as_string=False, element_name=nested_key, xmlns=xmlns))
+                    elif prop_info.concrete_type in (float, int):
+                        ElementTree.SubElement(nested_e, nested_key).text = str(j)
+                    elif prop_info.concrete_type is bool:
+                        ElementTree.SubElement(nested_e, nested_key).text = str(j).lower()
+                    else:
+                        # Assume type is str
+                        ElementTree.SubElement(nested_e, nested_key).text = str(j)
+            elif prop_info.is_enum:
+                ElementTree.SubElement(this_e, new_key).text = str(v.value)
+            elif not prop_info.is_primitive_type():
+                global_klass_name = f'{prop_info.concrete_type.__module__}.{prop_info.concrete_type.__name__}'
+                if global_klass_name in ObjectMetadataLibrary.klass_mappings:
+                    # Handle other Serializable Classes
+                    this_e.append(v.as_xml(as_string=False, element_name=new_key, xmlns=xmlns))
+                else:
+                    # Handle properties that have a type that is not a Python Primitive (e.g. int, float, str)
                     ElementTree.SubElement(this_e, new_key).text = str(v)
+            elif prop_info.concrete_type in (float, int):
+                ElementTree.SubElement(this_e, new_key).text = str(v)
+            elif prop_info.concrete_type is bool:
+                ElementTree.SubElement(this_e, new_key).text = str(v).lower()
+            else:
+                # Assume type is str
+                ElementTree.SubElement(this_e, new_key).text = str(v)
 
     if as_string:
         return ElementTree.tostring(this_e, 'unicode')
@@ -445,13 +478,24 @@ def _from_xml(cls: Type[_T], data: Union[TextIOWrapper, ElementTree.Element],
             else:
                 _data[decoded_k] = prop_info.concrete_type(child_e.text)
         else:
-            _data[decoded_k] = prop_info.concrete_type(child_e.text)
+            if prop_info.concrete_type == bool:
+                _data[decoded_k] = True if str(child_e.text) in (1, 'true') else False
+            else:
+                _data[decoded_k] = prop_info.concrete_type(child_e.text)
 
     logging.debug(f'Creating {cls} from {_data}')
 
     return cls(**_data)
 
 
+def _namespace_element_name(tag_name: str, xmlns: Optional[str]) -> str:
+    if tag_name.startswith('{'):
+        return tag_name
+    if xmlns:
+        return f'{{{xmlns}}}{tag_name}'
+    return tag_name
+
+
 class ObjectMetadataLibrary:
     """
     The core Class in ``serializable`` that is used to record all metadata about classes that you annotate for
@@ -462,6 +506,7 @@ class ObjectMetadataLibrary:
     _klass_property_attributes: Set[str] = set()
     _klass_property_names: Dict[str, Dict[SerializationType, str]] = {}
     _klass_property_types: Dict[str, Type[Any]] = {}
+    _klass_property_xml_sequence: Dict[str, int] = {}
     klass_mappings: Dict[str, 'ObjectMetadataLibrary.SerializableClass'] = {}
     klass_property_mappings: Dict[str, Dict[str, 'ObjectMetadataLibrary.SerializableProperty']] = {}
 
@@ -517,7 +562,8 @@ class ObjectMetadataLibrary:
 
         def __init__(self, *, prop_name: str, prop_type: Any, custom_names: Dict[SerializationType, str],
                      custom_type: Optional[Any] = None, is_xml_attribute: bool = False,
-                     xml_array_config: Optional[Tuple[XmlArraySerializationType, str]] = None) -> None:
+                     xml_array_config: Optional[Tuple[XmlArraySerializationType, str]] = None,
+                     xml_sequence: Optional[int] = None) -> None:
             self._name = prop_name
             self._custom_names = custom_names
             self._type_ = None
@@ -528,6 +574,7 @@ class ObjectMetadataLibrary:
             self._custom_type = custom_type
             self._is_xml_attribute = is_xml_attribute
             self._xml_array_config = xml_array_config
+            self._xml_sequence = xml_sequence or 100
 
             self._deferred_type_parsing = False
             self._parse_type(type_=prop_type)
@@ -575,6 +622,10 @@ class ObjectMetadataLibrary:
         def is_optional(self) -> bool:
             return self._is_optional
 
+        @property
+        def xml_sequence(self) -> int:
+            return self._xml_sequence
+
         def is_helper_type(self) -> bool:
             if inspect.isclass(self.custom_type):
                 return issubclass(self.custom_type, BaseHelper)
@@ -617,10 +668,10 @@ class ObjectMetadataLibrary:
                                     _k = _oml_sc.klass
 
                             if _k is None:
-                                self._type_ = type_
+                                self._type_ = type_  # type: ignore
                                 self._deferred_type_parsing = True
                                 ObjectMetadataLibrary.defer_property_type_parsing(
-                                    prop=self, klasses=[results.get("array_of")]
+                                    prop=self, klasses=[str(results.get("array_of"))]
                                 )
                                 return
 
@@ -655,11 +706,28 @@ class ObjectMetadataLibrary:
             if self._deferred_type_parsing:
                 self._deferred_type_parsing = False
 
+        def __eq__(self, other: Any) -> bool:
+            if isinstance(other, ObjectMetadataLibrary.SerializableProperty):
+                return hash(other) == hash(self)
+            return False
+
+        def __lt__(self, other: Any) -> bool:
+            if isinstance(other, ObjectMetadataLibrary.SerializableProperty):
+                return self.xml_sequence < other.xml_sequence
+            return NotImplemented
+
+        def __hash__(self) -> int:
+            return hash((
+                self.concrete_type, tuple(self.custom_names), self.custom_type, self.is_array, self.is_enum,
+                self.is_optional, self.is_xml_attribute, self.name, self.type_,
+                tuple(self.xml_array_config) if self.xml_array_config else None, self.xml_sequence
+            ))
+
         def __repr__(self) -> str:
             return f'<s.oml.SerializableProperty name={self.name}, custom_names={self.custom_names}, ' \
                    f'array={self.is_array}, enum={self.is_enum}, optional={self.is_optional}, ' \
                    f'c_type={self.concrete_type}, type={self.type_}, custom_type={self.custom_type}, ' \
-                   f'xml_attr={self.is_xml_attribute}>'
+                   f'xml_attr={self.is_xml_attribute}, xml_sequence={self.xml_sequence}>'
 
     @classmethod
     def defer_property_type_parsing(cls, prop: 'ObjectMetadataLibrary.SerializableProperty',
@@ -709,7 +777,8 @@ class ObjectMetadataLibrary:
                     is_xml_attribute=(qualified_property_name in ObjectMetadataLibrary._klass_property_attributes),
                     xml_array_config=ObjectMetadataLibrary._klass_property_array_config.get(
                         qualified_property_name, None
-                    )
+                    ),
+                    xml_sequence=ObjectMetadataLibrary._klass_property_xml_sequence.get(qualified_property_name, 100)
                 )
             })
 
@@ -723,7 +792,7 @@ class ObjectMetadataLibrary:
 
         # Handle any deferred Properties depending on this class
         if klass.__qualname__ in ObjectMetadataLibrary._deferred_property_type_parsing:
-            for _p in ObjectMetadataLibrary._deferred_property_type_parsing.get(klass.__qualname__):
+            for _p in ObjectMetadataLibrary._deferred_property_type_parsing.get(klass.__qualname__, {}):
                 _p.parse_type_deferred()
 
         return klass
@@ -751,6 +820,10 @@ class ObjectMetadataLibrary:
     def register_xml_property_attribute(cls, qual_name: str) -> None:
         cls._klass_property_attributes.add(qual_name)
 
+    @classmethod
+    def register_xml_property_sequence(cls, qual_name: str, sequence: int) -> None:
+        cls._klass_property_xml_sequence.update({qual_name: sequence})
+
     @classmethod
     def register_property_type_mapping(cls, qual_name: str, mapped_type: Any) -> None:
         cls._klass_property_types.update({qual_name: mapped_type})
@@ -870,3 +943,19 @@ def xml_name(name: str) -> Callable[[_F], _F]:
         return cast(_F, inner)
 
     return outer
+
+
+def xml_sequence(sequence: int) -> Callable[[_F], _F]:
+    def outer(f: _F) -> _F:
+        logger.debug(f'Registering {f.__module__}.{f.__qualname__} with XML sequence: {sequence}')
+        ObjectMetadataLibrary.register_xml_property_sequence(
+            qual_name=f'{f.__module__}.{f.__qualname__}', sequence=sequence
+        )
+
+        @functools.wraps(f)
+        def inner(*args: Any, **kwargs: Any) -> Any:
+            return f(*args, **kwargs)
+
+        return cast(_F, inner)
+
+    return outer
